@@ -10,17 +10,17 @@ import abc
 from typing import Any, Mapping, Optional
 
 import numpy as np
-from sklearn.base import BaseEstimator, TransformerMixin
 
-from ... import FDataGrid
 from ..._utils import _to_grid_points
-from ...representation._typing import GridPointsLike
+from ..._utils._sklearn_adapter import BaseEstimator, TransformerMixin
+from ...representation import FDataGrid
+from ...typing._base import GridPoints, GridPointsLike
+from ...typing._numpy import NDArrayFloat
 
 
 class _LinearSmoother(
-    abc.ABC,
-    BaseEstimator,  # type: ignore
-    TransformerMixin,  # type: ignore
+    BaseEstimator,
+    TransformerMixin[FDataGrid, FDataGrid, object],
 ):
     """Linear smoother.
 
@@ -28,6 +28,8 @@ class _LinearSmoother(
     ``hat_matrix`` to define the smoothing or 'hat' matrix.
 
     """
+    input_points_: GridPoints
+    output_points_: GridPoints
 
     def __init__(
         self,
@@ -40,7 +42,7 @@ class _LinearSmoother(
         self,
         input_points: Optional[GridPointsLike] = None,
         output_points: Optional[GridPointsLike] = None,
-    ) -> np.ndarray:
+    ) -> NDArrayFloat:
 
         # Use the fitted points if they are not provided
         if input_points is None:
@@ -58,7 +60,7 @@ class _LinearSmoother(
         self,
         input_points: GridPointsLike,
         output_points: GridPointsLike,
-    ) -> np.ndarray:
+    ) -> NDArrayFloat:
         pass
 
     def _more_tags(self) -> Mapping[str, Any]:
@@ -69,7 +71,7 @@ class _LinearSmoother(
     def fit(
         self,
         X: FDataGrid,
-        y: None = None,
+        y: object = None,
     ) -> _LinearSmoother:
         """Compute the hat matrix for the desired output points.
 
@@ -95,7 +97,7 @@ class _LinearSmoother(
     def transform(
         self,
         X: FDataGrid,
-        y: None = None,
+        y: object = None,
     ) -> FDataGrid:
         """Multiply the hat matrix with the function values to smooth them.
 
